import { world, system, ItemStack, EntityTypes } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import { mobCategories } from "./mobCategories.js";

// === 初期設定 ===
const defaultDropSettings = {
  friendly: { enabled: true, chance: 5, effectsEnabled: true },
  neutral: { enabled: false, chance: 5, effectsEnabled: true },
  hostile: { enabled: false, chance: 5, effectsEnabled: true },
  boss: { enabled: false, chance: 5, effectsEnabled: true },
};

let dropSettings = { ...defaultDropSettings };

// === DynamicProperty 用意 ===
const DP_KEY = "eggDropSettings";

world.beforeEvents.worldInitialize.subscribe(() => {
  loadSettings();
});

// === モブ撃破処理 ===
world.afterEvents.entityDie.subscribe((event) => {
  const dieEntity = event.deadEntity;
  const killer = event.damageSource?.damagingEntity;

  if (!killer || !dieEntity || !dieEntity.typeId.startsWith("minecraft:")) return;
  if (dieEntity.typeId === "minecraft:item") return;
  if (killer.typeId !== EntityTypes.get("minecraft:player").id) return;

  const mobId = dieEntity.typeId;
  const category = mobCategories[mobId];

  if (!category || !dropSettings[category]?.enabled) return;

  const chance = dropSettings[category].chance;
  if (Math.random() * 100 > chance) return;

  try {
    const eggId = `${mobId}_spawn_egg`;
    const item = new ItemStack(eggId, 1);
    dieEntity.dimension.spawnItem(item.clone(), dieEntity.location);
  } catch (e) {
    console.warn(`スポーン失敗: ${e.stack}`);
  }

  if (dropSettings[category].effectsEnabled) {
    showDropEffect(dieEntity.location, dieEntity.dimension);
  }
});

// === パーティクル演出 ===
function showDropEffect(location, dimension) {
  dimension.spawnParticle("minecraft:totem_particle", location);
}

// === 設定画面起動 ===
world.beforeEvents.itemUseOn.subscribe(async (event) => {
  const player = event.source;

  if (!player.isSneaking) return;
  if (event.itemStack.typeId !== "minecraft:egg") return;
  if (player.typeId !== EntityTypes.get("minecraft:player").id) return;
  if (!player.hasTag("op")) return;

  event.cancel = true;
  await showCategorySelectForm(player);
});

// === カテゴリ選択画面 ===
function showCategorySelectForm(player) {
  system.run(() => {
    const form = new ActionFormData()
      .title("スポーンエッグ設定")
      .body("設定するモブのカテゴリを選択してください")
      .button("友好的モブ")
      .button("中立的モブ")
      .button("敵対的モブ")
      .button("ボスモブ");

    form.show(player).then((res) => {
      if (res.canceled) return;

      const keys = ["friendly", "neutral", "hostile", "boss"];
      const category = keys[res.selection];
      if (!dropSettings[category]) {
        player.sendMessage("設定データが見つかりませんでした。");
        return;
      }

      showCategoryDetailForm(player, category);
    }).catch((err) => {
      console.warn("フォーム表示エラー:", err);
    });
  });
}

// === カテゴリ詳細フォーム ===
function showCategoryDetailForm(player, category) {
  const config = dropSettings[category];

  system.run(() => {
    const form = new ModalFormData()
      .title(`${categoryLabel(category)}モブの設定`)
      .toggle("スポーンエッグドロップの有効化", config.enabled)
      .slider("ドロップ確率（％）", 0, 100, 1, config.chance)
      .toggle("ドロップ時のエフェクトの有効化", config.effectsEnabled);

    form.show(player).then((res) => {
      if (res.canceled) return;

      config.enabled = res.formValues[0];
      const chance = parseFloat(res.formValues[1]);
      config.chance = isNaN(chance) ? 0 : Math.max(0, Math.min(chance, 100));
      config.effectsEnabled = res.formValues[2];

      saveSettings();
      player.sendMessage(`【スポーンエッグドロップ アドオン】${categoryLabel(category)}モブの設定を保存しました。`);
    }).catch((err) => {
      console.warn("フォーム詳細表示エラー:", err);
    });
  });
}

// === カテゴリ名表示用 ===
function categoryLabel(category) {
  switch (category) {
    case "friendly": return "友好的";
    case "neutral": return "中立的";
    case "hostile": return "敵対的";
    case "boss": return "ボス";
    default: return category;
  }
}

// === 設定の保存 ===
function saveSettings() {
  world.setDynamicProperty(DP_KEY, JSON.stringify(dropSettings));
}

// === 設定の読み込み ===
function loadSettings() {
  try {
    const raw = world.getDynamicProperty(DP_KEY);
    if (raw) {
      dropSettings = JSON.parse(raw);
    } else {
      saveSettings();
    }
  } catch (e) {
    console.warn("設定の読み込みに失敗しました。初期設定を使用します。");
    dropSettings = { ...defaultDropSettings };
    saveSettings(); // 念のため保存
  }
}
